<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  System.Debug
 *
 * @copyright   Copyright (C) 2005 - 2012 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_JEXEC') or die;

/**
 * Hidden Debug plugin
 *
 * @package     Joomla.Plugin
 * @subpackage  System.Debug
 * @since       1.5
 */
class plgSystemHiddenDebug extends JPlugin
{
	protected $linkFormat = '';

	/**
	 * Constructor.
	 *
	 * @param   object  &$subject  The object to observe
	 * @param   array   $config    An array that holds the plugin configuration
	 *
	 * @since 1.5
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		// Log the deprecated API.
		if ($this->params->get('log-deprecated'))
		{
			JLog::addLogger(array('text_file' => 'deprecated.php'), JLog::ALL, array('deprecated'));
		}

		// Only if debugging or language debug is enabled
		if (JDEBUG || JFactory::getApplication()->getCfg('debug_lang'))
		{
			JFactory::getConfig()->set('gzip', 0);
			ob_start();
			ob_implicit_flush(false);
		}

		$this->linkFormat = ini_get('xdebug.file_link_format');
	}

	/**
	 * Add the CSS for debug. We can't do this in the constructor because
	 * stuff breaks.
	 *
	 * @return  void
	 *
	 * @since   2.5
	 */
	public function onAfterDispatch()
	{
	}

	/**
	 * Show the debug info
	 *
	 * @since  1.6
	 */
	public function __destruct()
	{
		// Do not render if debugging or language debug is not enabled
		if (!JDEBUG && !JFactory::getApplication()->getCfg('debug_lang'))
		{
			return;
		}

		if (JFactory::getApplication()->isAdmin()) {
			return;
		}

		// Load the language
		$this->loadLanguage();

		// Capture output
		$contents = ob_get_contents();
		ob_end_clean();

		// No debug for Safari and Chrome redirection
		if (strstr(strtolower($_SERVER['HTTP_USER_AGENT']), 'webkit') !== false
			&& substr($contents, 0, 50) == '<html><head><meta http-equiv="refresh" content="0;')
		{
			echo $contents;
			return;
		}

		// Only render for HTML output
		if ('html' !== JFactory::getDocument()->getType())
		{
			echo $contents;
			return;
		}

		// If the user is not allowed to view the output then end here
		$filterGroups = (array) $this->params->get('filter_groups', null);

		if (!empty($filterGroups))
		{
			$userGroups = JFactory::getUser()->get('groups');

			if (!array_intersect($filterGroups, $userGroups))
			{
				echo $contents;
				return;
			}
		}

		// Load language file
		$this->loadLanguage('plg_system_hiddendebug');

		$html = '';

		$html .= '<!-- HIDDENDEBUG OUTPUT START: ' . "\n\n";


		$html .= JText::_('PLG_SYSTEM_HIDDENDEBUG_TITLE') . "\n\n";

		if (JDEBUG)
		{
			if ($this->params->get('profile', 1))
			{
				$html .= $this->display('profile_information');
			}

			$html .= "\n";

			if ($this->params->get('memory', 1))
			{
				$html .= $this->display('memory_usage');
			}

			$html .= "\n";

			if ($this->params->get('queries', 1))
			{
				$html .= $this->display('queries');
			}

			$html .= "\n";
		}

		$html .= 'HIDDENDEBUG OUTPUT FINISH: -->' . "\n";
		echo str_replace('</body>', $html . '</body>', $contents);
	}

	/**
	 * General display method.
	 *
	 * @param   string  $item    The item to display
	 * @param   array   $errors  Errors occured during execution
	 *
	 * @return  string
	 *
	 * @since   2.5
	 */
	protected function display($item, array $errors = array())
	{
		$title = JText::_('PLG_SYSTEM_HIDDENDEBUG_' . strtoupper($item));

		$status = '';

		if (count($errors))
		{
			$status = ' dbgerror';
		}

		$fncName = 'display' . ucfirst(str_replace('_', '', $item));

		if (!method_exists($this, $fncName))
		{
			return __METHOD__ . ' -- Unknown method: ' . $fncName . '<br />';
		}

		$html = '';

		$html .= $title . "\n";

		$html .= $this->$fncName();

		return $html;
	}

	/**
	 * Display profile information.
	 *
	 * @return  string
	 *
	 * @since   2.5
	 */
	protected function displayProfileInformation()
	{
		$html = '';

		foreach (JProfiler::getInstance('Application')->getBuffer() as $mark)
		{
			$html .= str_replace(array('<code>', '</code>'), '', $mark) . "\n";
		}

		return $html;
	}

	/**
	 * Display memory usage
	 *
	 * @return  string
	 *
	 * @since   2.5
	 */
	protected function displayMemoryUsage()
	{
		$html = '';

		$bytes = JProfiler::getInstance('Application')->getMemory();

		$html .= JHtml::_('number.bytes', $bytes);
		$html .= ' (' . number_format($bytes) . ' Bytes)' . "\n";

		return $html;
	}

	/**
	 * Display logged queries.
	 *
	 * @return  string
	 *
	 * @since   2.5
	 */
	protected function displayQueries()
	{
		$db	= JFactory::getDbo();

		$log = $db->getLog();

		if ( ! $log)
		{
			return;
		}

		$html = '';

		$queryCount = (int) $db->getCount();
		$html .= JText::sprintf('PLG_SYSTEM_HIDDENDEBUG_QUERIES_LOGGED',  $queryCount) . "\n";

		$notificationThreshold = (int) $this->params->get('querynotificationthreshold', 0);
		if (!empty($notificationThreshold) && $queryCount >= $notificationThreshold)
		{
			$from = JFactory::getConfig()->getValue('config.mailfrom');
			$fromname = JFactory::getConfig()->getValue('config.fromname');

			// Send emails to the people the plugin is configured to notify:
			$recipient = array();
			$emailsToNotify = $this->params->get('notificationemails', '');
			$emailsToNotify = str_replace(' ', '', $emailsToNotify);
			if (!empty($emailsToNotify))
			{
				$emailsToNotify = explode(',', $emailsToNotify);
				$recipient = $emailsToNotify;
			}

			$subject = 'Query Threshold Notification (' . $queryCount . ' queries) for ' . JFactory::getURI()->toString();
			$body = $this->createBody($queryCount);
			$isHTML = 1;
			$cc = null;
			$bcc = null;

			if (!empty($recipient))
			{
				JUtility::sendMail($from, $fromname, $recipient, $subject, $body, $isHTML, $cc, $bcc);
			}
		}

		$selectQueryTypeTicker = array();
		$otherQueryTypeTicker = array();

		foreach ($log as $k => $sql)
		{
			// Start Query Type Ticker Additions
			$fromStart = stripos($sql, 'from');
			$whereStart = stripos($sql, 'where', $fromStart);

			if ($whereStart === false)
			{
				$whereStart = stripos($sql, 'order by', $fromStart);
			}

			if ($whereStart === false)
			{
				$whereStart = strlen($sql) - 1;
			}

			$fromString = substr($sql, 0, $whereStart);
			$fromString = str_replace("\t", " ", $fromString);
			$fromString = str_replace("\n", " ", $fromString);
			$fromString = trim($fromString);

			// Initialize the select/other query type counts the first time:
			if (!isset($selectQueryTypeTicker[$fromString]))
			{
				$selectQueryTypeTicker[$fromString] = 0;
			}

			if (!isset($otherQueryTypeTicker[$fromString]))
			{
				$otherQueryTypeTicker[$fromString] = 0;
			}

			// Increment the count:
			if (stripos($sql, 'select') === 0)
			{
				$selectQueryTypeTicker[$fromString] = $selectQueryTypeTicker[$fromString] + 1;
				unset($otherQueryTypeTicker[$fromString]);
			}
			else
			{
				$otherQueryTypeTicker[$fromString] = $otherQueryTypeTicker[$fromString] + 1;
				unset($selectQueryTypeTicker[$fromString]);
			}
		}

		// Get the totals for the query types:
		$totalSelectQueryTypes = count($selectQueryTypeTicker);
		$totalOtherQueryTypes = count($otherQueryTypeTicker);
		$totalQueryTypes = $totalSelectQueryTypes + $totalOtherQueryTypes;

		$html .= JText::sprintf('PLG_SYSTEM_HIDDENDEBUG_QUERY_TYPES_LOGGED', $totalQueryTypes) . "\n";

		return $html;
	}

	protected function createBody($queries = 0)
	{
		$sitename = JFactory::getConfig()->getValue( 'config.sitename' );
		$siteroot = JFactory::getURI()->root();
		$uri = JFactory::getURI()->toString();

		$body = '';

		$body .= '<html><head></head><body style="font-family: Calibri,Tahoma,Arial;">';

		$body .= '<p>';
		$body .= 'This notification is for: <a href="' . $siteroot . '">' . $siteroot . '</a>.<br />';
		$body .= '</p>';

		$body .= '<p>';
		$body .= 'The following page: ';
		$body .= '<a href="' . $uri . '">' . $uri . '</a>.<br />';
		$body .= '</p>';

		$body .= '<p>';
		$body .= 'Is currently executing ' . $queries . ' queries, which is above the threshold you have set in the Hidden Debug plugin. You may want to consider caching this specific page.';
		$body .= '</p>';

		$body .= '</body></html>';

		return $body;
	}

}
